@@ -31,8 +31,8 @@ struct ServiceInternal {
     derive(Serialize, Deserialize),
     serde(crate = "serde_cr")
 )]
-#[derive(Clone, Debug, Eq, Hash, PartialEq)]
-pub struct PeripheralId(BDAddr);
+#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+pub struct PeripheralId(pub(crate) DeviceId);
 
 /// Implementation of [api::Peripheral](crate::api::Peripheral).
 #[derive(Clone, Debug)]
@@ -48,7 +48,7 @@ impl Peripheral {
         Peripheral {
             session,
             device: device.id,
-            mac_address: (&device.mac_address).into(),
+            mac_address: device.mac_address.into(),
             services: Arc::new(Mutex::new(HashMap::new())),
         }
     }
@@ -88,7 +88,7 @@ impl Peripheral {
 #[async_trait]
 impl api::Peripheral for Peripheral {
     fn id(&self) -> PeripheralId {
-        PeripheralId(self.address())
+        PeripheralId(self.device.to_owned())
     }
 
     fn address(&self) -> BDAddr {
@@ -98,7 +98,7 @@ impl api::Peripheral for Peripheral {
     async fn properties(&self) -> Result<Option<PeripheralProperties>> {
         let device_info = self.device_info().await?;
         Ok(Some(PeripheralProperties {
-            address: (&device_info.mac_address).into(),
+            address: device_info.mac_address.into(),
             address_type: Some(device_info.address_type.into()),
             local_name: device_info.name,
             tx_power_level: device_info.tx_power,
@@ -239,15 +239,15 @@ impl From<WriteType> for bluez_async::WriteType {
     }
 }
 
-impl From<&MacAddress> for BDAddr {
-    fn from(mac_address: &MacAddress) -> Self {
-        mac_address.to_string().parse().unwrap()
+impl From<MacAddress> for BDAddr {
+    fn from(mac_address: MacAddress) -> Self {
+        <[u8; 6]>::into(mac_address.into())
     }
 }
 
-impl From<&MacAddress> for PeripheralId {
-    fn from(mac_address: &MacAddress) -> Self {
-        PeripheralId(BDAddr::from(mac_address))
+impl From<DeviceId> for PeripheralId {
+    fn from(device_id: DeviceId) -> Self {
+        PeripheralId(device_id)
     }
 }
 
