@@ -1,6 +1,6 @@
 use super::internal::{run_corebluetooth_thread, CoreBluetoothEvent, CoreBluetoothMessage};
-use super::peripheral::Peripheral;
-use crate::api::{BDAddr, Central, CentralEvent};
+use super::peripheral::{Peripheral, PeripheralId};
+use crate::api::{BDAddr, Central, CentralEvent, ScanFilter};
 use crate::common::adapter_manager::AdapterManager;
 use crate::{Error, Result};
 use async_trait::async_trait;
@@ -8,22 +8,17 @@ use futures::channel::mpsc::{self, Sender};
 use futures::sink::SinkExt;
 use futures::stream::{Stream, StreamExt};
 use log::*;
-use std::convert::{TryFrom, TryInto};
 use std::pin::Pin;
+use std::sync::Arc;
 use tokio::task;
 
 /// Implementation of [api::Central](crate::api::Central).
 #[derive(Clone, Debug)]
 pub struct Adapter {
-    manager: AdapterManager<Peripheral>,
+    manager: Arc<AdapterManager<Peripheral>>,
     sender: Sender<CoreBluetoothMessage>,
 }
 
-pub(crate) fn uuid_to_bdaddr(uuid: &str) -> BDAddr {
-    let b: [u8; 6] = uuid.as_bytes()[0..6].try_into().unwrap();
-    BDAddr::try_from(b).unwrap()
-}
-
 impl Adapter {
     pub(crate) async fn new() -> Result<Self> {
         let (sender, mut receiver) = mpsc::channel(256);
@@ -41,39 +36,36 @@ impl Adapter {
             ));
         }
         debug!("Adapter connected");
-        let manager = AdapterManager::default();
+        let manager = Arc::new(AdapterManager::default());
 
         let manager_clone = manager.clone();
         let adapter_sender_clone = adapter_sender.clone();
         task::spawn(async move {
             while let Some(msg) = receiver.next().await {
                 match msg {
-                    CoreBluetoothEvent::DeviceDiscovered(uuid, name, event_receiver) => {
-                        // TODO Gotta change uuid into a BDAddr for now. Expand
-                        // library identifier type. :(
-                        let id = uuid_to_bdaddr(&uuid.to_string());
-                        manager_clone.add_peripheral(
-                            id,
-                            Peripheral::new(
-                                uuid,
-                                name,
-                                manager_clone.clone(),
-                                event_receiver,
-                                adapter_sender_clone.clone(),
-                            ),
-                        );
-                        manager_clone.emit(CentralEvent::DeviceDiscovered(id));
+                    CoreBluetoothEvent::DeviceDiscovered {
+                        uuid,
+                        name,
+                        event_receiver,
+                    } => {
+                        manager_clone.add_peripheral(Peripheral::new(
+                            uuid,
+                            name,
+                            Arc::downgrade(&manager_clone),
+                            event_receiver,
+                            adapter_sender_clone.clone(),
+                        ));
+                        manager_clone.emit(CentralEvent::DeviceDiscovered(uuid.into()));
                     }
-                    CoreBluetoothEvent::DeviceUpdated(uuid, name) => {
-                        let id = uuid_to_bdaddr(&uuid.to_string());
-                        if let Some(mut entry) = manager_clone.peripheral_mut(id) {
+                    CoreBluetoothEvent::DeviceUpdated { uuid, name } => {
+                        let id = uuid.into();
+                        if let Some(entry) = manager_clone.peripheral_mut(&id) {
                             entry.value().update_name(&name);
                             manager_clone.emit(CentralEvent::DeviceUpdated(id));
                         }
                     }
-                    CoreBluetoothEvent::DeviceLost(uuid) => {
-                        let id = uuid_to_bdaddr(&uuid.to_string());
-                        manager_clone.emit(CentralEvent::DeviceDisconnected(id));
+                    CoreBluetoothEvent::DeviceDisconnected { uuid } => {
+                        manager_clone.emit(CentralEvent::DeviceDisconnected(uuid.into()));
                     }
                     _ => {}
                 }
@@ -95,10 +87,10 @@ impl Central for Adapter {
         Ok(self.manager.event_stream())
     }
 
-    async fn start_scan(&self) -> Result<()> {
+    async fn start_scan(&self, filter: ScanFilter) -> Result<()> {
         self.sender
             .to_owned()
-            .send(CoreBluetoothMessage::StartScanning)
+            .send(CoreBluetoothMessage::StartScanning { filter })
             .await?;
         Ok(())
     }
@@ -115,10 +107,8 @@ impl Central for Adapter {
         Ok(self.manager.peripherals())
     }
 
-    async fn peripheral(&self, address: BDAddr) -> Result<Peripheral> {
-        self.manager
-            .peripheral(address)
-            .ok_or(Error::DeviceNotFound)
+    async fn peripheral(&self, id: &PeripheralId) -> Result<Peripheral> {
+        self.manager.peripheral(id).ok_or(Error::DeviceNotFound)
     }
 
     async fn add_peripheral(&self, _address: BDAddr) -> Result<Peripheral> {
@@ -126,4 +116,9 @@ impl Central for Adapter {
             "Can't add a Peripheral from a BDAddr".to_string(),
         ))
     }
+
+    async fn adapter_info(&self) -> Result<String> {
+        // TODO: Get information about the adapter.
+        Ok("CoreBluetooth".to_string())
+    }
 }
