@@ -16,7 +16,7 @@
 // This file may not be copied, modified, or distributed except
 // according to those terms.
 
-use objc::runtime::Object;
+use cocoa::base::id;
 use std::slice;
 use uuid::Uuid;
 
@@ -26,7 +26,7 @@ use super::framework::ns;
 pub mod core_bluetooth;
 pub mod nsstring;
 
-pub fn nsdata_to_vec(data: *mut Object) -> Vec<u8> {
+pub fn nsdata_to_vec(data: id) -> Vec<u8> {
     let length = ns::data_length(data);
     if length == 0 {
         return vec![];
@@ -35,7 +35,7 @@ pub fn nsdata_to_vec(data: *mut Object) -> Vec<u8> {
     unsafe { slice::from_raw_parts(bytes, length as usize).to_vec() }
 }
 
-pub fn nsuuid_to_uuid(uuid: *mut Object) -> Uuid {
+pub fn nsuuid_to_uuid(uuid: id) -> Uuid {
     let uuid_nsstring = ns::uuid_uuidstring(uuid);
     nsstring_to_string(uuid_nsstring).unwrap().parse().unwrap()
 }
